import torch
import torch.nn.functional as F
from torch.autograd import Variable
import numpy as np


def _l2_normalize(d):
    d_reshaped = d.view(d.shape[0], -1, *(1 for _ in range(d.dim() - 2)))
    d /= torch.norm(d_reshaped, dim=1, keepdim=True) + 1e-8
    return d


def vat_loss(model, lossfn, ul_x, ul_y, xi=1e-6, eps=2.5, num_iters=1):

    # generate a random unit vector d using iid Gaussian distribution
    d = torch.rand_like(ul_x)
    d =  _l2_normalize(d)
    # use power iteration method to calculate r_adv over num_iters (default is 1)
    for i in range(num_iters):
        d.requires_grad_()
        y_hat = F.softmax(model(ul_x + xi*d),1)
        # KL Divergence loss
        delta_kl = lossfn(F.log_softmax(ul_y,1), y_hat)
        # get the gradient of d w.r.t loss
        delta_kl.backward()
        # get the normalized unit vector d
        d = _l2_normalize(d.grad)
        # flush the grads to not add gradients over iterations
        model.zero_grad()

	# calculate r_adv
    r_adv = eps*d
    # return loss, but ignore the derivative of r_adv w.r.t model parameters
    y_hat = F.softmax(model(ul_x + r_adv.detach()),1)
    delta_kl = lossfn(F.log_softmax(ul_y.detach(),1), y_hat)
    return delta_kl
